searchNodes=[{"doc":"NimbleTOTP is a tiny library for Two-factor authentication (2FA) that allows developers to implement Time-Based One-Time Passwords (TOTP) for their applications. Two-factor authentication (2FA) The concept of 2FA is quite simple. It's an extra layer of security that demands a user to provide two pieces of evidence (factors) to the authentication system before access can be granted. One way to implement 2FA is to generate a random secret for the user and whenever the system needs to perform a critical action it will ask the user to enter a validation code. This validation code is a Time-Based One-Time Password (TOTP) based on the user's secret and can be provided by an authentication app like Google Authenticator or Authy, which should be previously installed and configured on a compatible device, e.g. a smartphone. Note: A critical action can mean different things depending on the application. For instance, while in a banking system the login itself is already considered a critical action, in other systems a user may be allowed to log in using just the password and only when trying to update critical data (e.g. its profile) 2FA will be required. Using NimbleTOTP In order to allow developers to implement 2FA, NimbleTOTP provides functions to: Generate secrets composed of random bytes. Generate URIs to be encoded in a QR Code. Generate Time-Based One-Time Passwords based on a secret. Generating the secret The first step to set up 2FA for a user is to generate (and later persist) its random secret. You can achieve that using NimbleTOTP.secret/1 . Example: secret = NimbleTOTP . secret ( ) #=&gt; &lt;&lt;178, 117, 46, 7, 172, 202, 108, 127, 186, 180, ...&gt;&gt; By default, a binary with 20 random bytes is generated per the HOTP RFC . Generating URIs for QR Code Before persisting the secret, you need to make sure the user has already configured the authentication app in a compatible device. The most common way to do that is to generate a QR Code that can be read by the app. You can use NimbleTOTP.otpauth_uri/3 along with eqrcode to generate the QR code as SVG . Example: uri = NimbleTOTP . otpauth_uri ( &quot;Acme:alice&quot; , secret , issuer : &quot;Acme&quot; ) #=&gt; &quot;otpauth://totp/Acme:alice?secret=MFRGGZA&amp;issuer=Acme&quot; uri |&gt; EQRCode . encode ( ) |&gt; EQRCode . svg ( ) #=&gt; &quot;&lt;?xml version=\\\\&quot;1.0\\\\&quot; standalone=\\\\&quot;yes\\\\&quot;?&gt;\\\\n&lt;svg version=\\\\&quot;1.1\\\\&quot; ... Generating a Time-Based One-Time Password After successfully reading the QR Code, the app will start generating a different 6 digit code every 30s . You can compute the verification code with: NimbleTOTP . verification_code ( secret ) #=&gt; &quot;569777&quot; The code can be validated using the valid?/3 function. Example: NimbleTOTP . valid? ( secret , &quot;569777&quot; ) #=&gt; true NimbleTOTP . valid? ( secret , &quot;012345&quot; ) #=&gt; false After validating the code, you can finally persist the user's secret so you use it later whenever you need to authorize any critical action using 2FA. Preventing codes from being reused The TOTP RFC requires that a code can only be used once. This is a security feature that prevents codes from being reused. To ensure codes are only considered valid if they have not been used, you need to keep track of the last time the user entered a TOTP code. NimbleTOTP . valid? ( user . totp_secret , code , since : user . last_totp_at ) Assuming the code itself is valid for the given secret, if since is nil , the code will be considered valid. If since is given, it will not allow codes in the same time period (30 seconds by default) to be reused. The user will have to wait for the next code to be generated.","ref":"NimbleTOTP.html","title":"NimbleTOTP","type":"module"},{"doc":"Generate the uri to be encoded in the QR code. Examples iex&gt; NimbleTOTP . otpauth_uri ( &quot;Acme:alice&quot; , &quot;abcd&quot; , issuer : &quot;Acme&quot; ) &quot;otpauth://totp/Acme:alice?secret=MFRGGZA&amp;issuer=Acme&quot;","ref":"NimbleTOTP.html#otpauth_uri/3","title":"NimbleTOTP.otpauth_uri/3","type":"function"},{"doc":"Generate a binary composed of random bytes. The number of bytes is defined by the size argument. Default is 20 per the HOTP RFC . Examples NimbleTOTP . secret ( ) #=&gt; &lt;&lt;178, 117, 46, 7, 172, 202, 108, 127, 186, 180, ...&gt;&gt;","ref":"NimbleTOTP.html#secret/1","title":"NimbleTOTP.secret/1","type":"function"},{"doc":"Checks if the given otp code matches the secret. Options :time - The time (either %NaiveDateTime{} , %DateTime{} or unix format) to be used. Default is System.os_time(:second) :since - The last time the secret was used, see &quot;Preventing TOTP code reuse&quot; next :period - The period (in seconds) in which the code is valid. Default is 30 . Preventing TOTP code reuse The :since option can be used to prevent TOTP codes from being reused. When set to the time when the last code was entered, the code generated from within that period is no longer considered valid. Periods are counted from the Unix epoch. This means a user may have to wait, in the worst case scenario, for the duration of :period before they can enter a valid code again. This implementation meets the TOTP RFC requirements. Grace period In some cases it is preferable to allow the user more time to validate the code than the initial period (mostly 30 seconds), the so-called grace period. Although this library does not support this out of the box, you can achieve the same functionality by using the :time option. def valid_code? ( secret , otp ) do time = System . os_time ( :second ) NimbleTOTP . valid? ( secret , otp , time : time ) or NimbleTOTP . valid? ( secret , otp , time : time - 30 ) end In this example by validating first against the current time, but also against 30 seconds ago, we allow the previous code, to be still valid.","ref":"NimbleTOTP.html#valid?/3","title":"NimbleTOTP.valid?/3","type":"function"},{"doc":"Generate Time-Based One-Time Password. Options :time - The time (either %NaiveDateTime{} , %DateTime{} or unix format) to be used. Default is System.os_time(:second) :period - The period (in seconds) in which the code is valid. Default is 30 . Examples NimbleTOTP . verification_code ( secret ) #=&gt; &quot;569777&quot;","ref":"NimbleTOTP.html#verification_code/2","title":"NimbleTOTP.verification_code/2","type":"function"},{"doc":"Options for verification_code/2 and valid?/3 .","ref":"NimbleTOTP.html#t:option/0","title":"NimbleTOTP.option/0","type":"type"},{"doc":"Unix time in seconds, t:DateTime.t() or t:NaiveDateTime.t() .","ref":"NimbleTOTP.html#t:time/0","title":"NimbleTOTP.time/0","type":"type"},{"doc":"Options for valid?/3 .","ref":"NimbleTOTP.html#t:validate_option/0","title":"NimbleTOTP.validate_option/0","type":"type"}]